{% extends "allianceauth/base.html" %}

{% load bootstrap %}
{% load i18n %}

{% block page_title %}{% translate "SRP Request" %}{% endblock page_title %}

{% block content %}
    <h1 class="page-header text-center">{% translate "Create SRP Request" %}</h1>

    <div class="container-fluid">
        <div class="col-md-4 col-md-offset-4">
            <div class="row">
                <form class="form-signin" role="form" action="" method="POST">
                    {% csrf_token %}
                    {{ form|bootstrap }}
                    <br>
                    <button class="btn btn-lg btn-primary btn-block" type="submit">{% translate "Create SRP Request" %}
                    </button>
                </form>
            </div>
        </div>
    </div>
{% endblock content %}
