/* General DataTables styles
------------------------------------------------------------------------------------- */
@media all {
    .dataTables_wrapper > .form-inline {
        margin-bottom: 0.625rem;
    }

    .dataTables_wrapper > .form-inline select {
        margin-bottom: 0.625rem;
        margin-right: 0.625rem;
    }

    .dataTables_wrapper > .form-inline select:last-child {
        margin-right: 0;
    }

    .dataTables_length select {
        margin: 0 0.625rem;
    }
}

@media (min-width: 768px) {
    .dataTables_wrapper > .form-inline select:first-child {
        margin-left: 0.625rem;
    }
}

/* App: ckEditor
------------------------------------------------------------------------------------- */
@media all {
    .cke_wysiwyg_frame,
    .cke_wysiwyg_div,
    .cke_inner {
        background: transparent !important;
    }
}

/* App: Fittings
------------------------------------------------------------------------------------- */
@media all {
    .aa-fittings-add-dropdown {
        margin-bottom: 0.5em;
    }
}

@media all and (min-width: 768px) {
    .aa-fittings-add-dropdown {
        margin-bottom: 0;
        margin-top: 0.5em;
    }
}

/* App: AA Blueprints
------------------------------------------------------------------------------------- */
@media all {
    .template-light-mode .aa-blueprints .table-striped > tbody > tr.tr-group,
    .template-light-mode .aa-blueprints .tr-group {
        background-color: var(--background-color-navigation-default-hover) !important;
        font-weight: bold;
    }
}

/* App: AA Memberaudit
------------------------------------------------------------------------------------- */
@media all {
    .aa-memberaudit {
        margin-left: 0;
    }

    .template-light-mode .aa-memberaudit .sidebar_character > a,
    .template-dark-mode .aa-memberaudit .sidebar_character > a {
        color: var(--white);
    }

    .template-light-mode .aa-memberaudit .sidebar_character:hover > a,
    .template-light-mode .aa-memberaudit .sidebar_character.active > a,
    .template-dark-mode .aa-memberaudit .sidebar_character:hover > a,
    .template-dark-mode .aa-memberaudit .sidebar_character.active > a {
        background-color: var(--background-color-navigation-default-hover);
    }

    .template-light-mode .aa-memberaudit .nav-tabs > li.active > a,
    .template-dark-mode .aa-memberaudit .nav-tabs > li.active > a {
        background-color: var(--background-color-navigation-default-hover) !important;
        color: var(--white) !important;
    }

    .template-light-mode .aa-memberaudit .table > tbody > .table-group > td {
        background-color: var(--background-color-navigation-default-hover);
        color: var(--white);
        font-weight: bold;
    }

    .template-light-mode .aa-memberaudit .cards_container .card,
    .template-dark-mode .aa-memberaudit .cards_container .card {
        background-color: rgba(100 100 100 / 50%);
    }

    .aa-memberaudit .cards_container {
        margin: 0 0 2rem;
    }
}

/* App: AA Sovtimer
------------------------------------------------------------------------------------- */
@media all {
    .aa-sovtimer {
        margin-left: 0;
    }

    .aa-sovtimer .table > tbody > tr > td {
        vertical-align: middle;
    }

    .aa-sovtimer .aa-sovtimer-trend {
        cursor: default;
        display: inline-block;
        left: 0.3125rem;
        margin-right: 0.625rem;
        position: relative;
        top: 0.34375rem;
    }

    .aa-sovtimer .aa-sov-timer-zkb-icon {
        margin-left: 0.5rem;
    }

    .aa-sovtimer .aa-sov-timer-zkb-icon > img {
        position: relative;
        top: -2px;
    }

    .aa-sovtimer .aa-sovtimer-campaigns tr td:last-child {
        padding-top: 0;
        white-space: nowrap;
    }

    .aa-sovtimer .aa-sovtimer-timer-elapsed {
        color: rgb(253 126 20);
        text-shadow: 1px 1px 1px rgba(255 0 0 / 30%);
    }
}

/* App: AA Structures
------------------------------------------------------------------------------------- */
@media all {
    .template-light-mode .nav-tabs > li.active > a {
        background-color: var(--background-color-navigation-default-hover) !important;
        color: var(--white) !important;
    }
}

/* App: Corpstat
------------------------------------------------------------------------------------- */
@media all {
    .corpstat-corp-dropdown.dropdown-menu > li > a {
        padding: 3px 36px 3px 20px;
    }
}

/* App: AA Fleet Pings
------------------------------------------------------------------------------------- */
@media all {
    .aa-fleetpings-pingtext {
        top: 3.6875rem;
    }
}

/* App: AA Discord Announcements
------------------------------------------------------------------------------------- */
@media all {
    .aa-discord-announcements-announcementtext {
        top: 3.6875rem;
    }
}

/* App: Another Fleet Activity Tracker (AFAT)
------------------------------------------------------------------------------------- */
@media all {
    .allianceauth-afat {
        margin-left: 0;
    }
}

/* App: AA SRP
------------------------------------------------------------------------------------- */
@media all {
    .aa-srp {
        margin-left: 0;
    }

    .srp-request-payout-amount-editable .srp-payout-amount {
        border-bottom: dashed 1px rgb(0 136 204);
        cursor: pointer;
        display: block;
    }

    .srp-request-status-rejected .srp-request-payout-amount-editable .srp-payout-amount {
        border-bottom: dashed 1px rgb(106 100 100);
        cursor: pointer;
        display: block;
    }

    .srp-request-payout-amount-editable .srp-payout-amount-changed {
        border-bottom: dashed 1px rgb(204 71 0) !important;
        cursor: pointer;
        display: block;
    }

    .modal-srp-details-request-history .request-history-comment {
        margin-top: 1rem;
        border-bottom: 1px solid rgb(28 30 34);
    }
}

/* App: AA Bulletin Board
------------------------------------------------------------------------------------- */
@media all {
    .aa-bulletin-board {
        margin-left: 0;
    }

    /* SumoSelect
    --------------------------------------------------------------------------------- */
    .aa-bulletin-board .SumoSelect {
        color: rgb(54 54 54);
    }

    .aa-bulletin-board .SumoSelect > .CaptionCont > span.placeholder {
        color: var(--background-color-navigation-default-hover);
    }
}

/* App: AA Forum
------------------------------------------------------------------------------------- */
@media all {
    .aa-forum {
        margin-left: 0;
    }

    .aa-forum .aa-forum-header {
        margin-bottom: 1rem;
    }

    .aa-forum .aa-forum-admin-panel-aa-forum-category .panel-body {
        min-height: auto;
    }

    .aa-forum .boards-sortable > li {
        background-color: var(--background-panel-header-default);
    }

    .aa-forum .panel-aa-forum-topic-list {
        margin: 1.25rem 0;
    }

    .aa-forum .aa-forum-topic-row .aa-forum-topic:hover,
    .aa-forum .panel-aa-forum-category .aa-forum-board:hover {
        background-color: rgba(73 81 90 / 80%);
    }

    .aa-forum .aa-forum-topic-locked .aa-forum-topic:hover {
        background-color: rgba(185 199 219 / 25%);
    }

    .aa-forum .aa-forum-topic-sticky .aa-forum-topic:hover {
        background-color: rgba(255 220 104 / 25%);
    }

    .aa-forum .aa-forum-message-body {
        border-top: 1px solid rgba(200 200 200 / 25%);
    }

    .aa-forum .label-aa-forum-topic-new-message {
        margin-left: 0.5rem;
    }

    .aa-forum .aa-forum-breadcrumb ul {
        margin: 0;
    }

    .aa-forum .aa-forum-breadcrumb ul li {
        padding: 0.35rem 0.3rem 0.35rem 0;
    }

    .aa-forum .aa-forum-legend {
        padding: 0.25rem 0;
    }

    .aa-forum .btn-aa-forum-topic-moderation {
        border: none;
    }

    /* SumoSelect
    --------------------------------------------------------------------------------- */
    .aa-forum .SumoSelect {
        color: rgb(54 54 54);
    }

    .aa-forum .SumoSelect > .CaptionCont > span.placeholder {
        color: var(--background-color-navigation-default-hover);
    }

    .aa-forum .aa-forum-search-results-total-number {
        font-size: 1.25rem;
        font-weight: bolder;
        margin-bottom: 2rem;
    }

    .aa-forum .aa-forum-search-result-counter {
        font-size: 2rem;
        font-weight: bolder;
        margin-right: 1rem;
        min-width: 50px;
        padding: 1rem;
    }

    .aa-forum #aa-forum-form-search-menu {
        padding: 10px 0;
    }

    .aa-forum .aa-forum-breadcrumb {
        padding: 1.3rem 1rem;
    }

    .aa-forum .aa-forum-legend > span {
        margin-right: 3rem;
    }
}

@media all and (min-width: 768px) {
    .aa-forum #aa-forum-form-search-menu {
        padding: 0;
    }

    .aa-forum .aa-forum-breadcrumb ul li:first-child {
        padding-left: 0;
    }
}
