@media all {
    html {
        background: transparent;
        background-size: cover;
        height: 100%;
    }

    body {
        background-color: var(--background-color-light-transparency);
        background-image: none;
        min-height: 100%;
        padding: 1rem;
    }

    p {
        margin: 0 0 1rem;
    }
}
