{% extends 'allianceauth/base.html' %}
{% load i18n %}

{% block page_title %}{% translate "Mumble Temp Links" %}{% endblock %}

{% block content %}
    <h1 class="page-header text-center">{% translate "Mumble Temporary Links" %}</h1>
    {% block create %}{% endblock %}

    {% block admin %}{% endblock %}
{% endblock %}
