{% load static %}

<link rel="apple-touch-icon" sizes="180x180" href="{% static 'tnnt_templates/favicons/apple-touch-icon.png' %}">
<link rel="icon" type="image/png" sizes="16x16" href="{% static 'tnnt_templates/favicons/favicon-16x16.png' %}">
<link rel="icon" type="image/png" sizes="32x32" href="{% static 'tnnt_templates/favicons/favicon-32x32.png' %}">
<link rel="icon" type="image/png" sizes="96x96" href="{% static 'tnnt_templates/favicons/favicon-96x96.png' %}">
<link rel="icon" type="image/png" sizes="192x192" href="{% static 'tnnt_templates/favicons/android-chrome-192x192.png' %}">
<link rel="manifest" href="{% static 'tnnt_templates/favicons/site.webmanifest' %}">
<link rel="mask-icon" href="{% static 'tnnt_templates/favicons/safari-pinned-tab.svg' %}" color="#5bbad5">
<link rel="shortcut icon" href="{% static 'tnnt_templates/favicons/favicon.ico' %}">

<meta name="msapplication-TileColor" content="#000000">
<meta name="msapplication-TileImage" content="{% static 'tnnt_templates/favicons/mstile-144x144.png' %}">
<meta name="msapplication-config" content="{% static 'tnnt_templates/favicons/browserconfig.xml' %}">
<meta name="theme-color" content="#ffffff">
