{% extends "allianceauth/base.html" %}
{% load bootstrap %}
{% load i18n %}

{% block page_title %}{% blocktrans with service_name=view.service_name|title %}{{ service_name }} Password Change{% endblocktrans %}{% endblock page_title %}

{% block content %}
    <div class="clearfix">
        <h1 class="page-header text-center">{% blocktrans with service_name=view.service_name|title %}Set {{service_name}} Password{% endblocktrans %}</h1>
        <div class="container-fluid">
            <div class="col-md-4 col-md-offset-4">
                <div class="row">
                    <form class="form-signin" role="form" action="" method="POST" onsubmit="submitbutton.disabled = true; return true;">
                        {% csrf_token %}
                        {{ form|bootstrap }}
                        <br>
                        <button class="btn btn-lg btn-primary btn-block" name="submitbutton" type="submit">{% translate "Set Password" %}</button>
                    </form>
                </div>
            </div>
        </div>
    </div>

{% endblock content %}
