/* Hide placeholders in form elements when they are being focused
------------------------------------------------------------------------------------- */
@media all {
    ::placeholder {
        opacity: 1;
        transition: opacity 0.5s;
    }

    *:focus::placeholder {
        opacity: 0;
    }
}

/* Defaults
------------------------------------------------------------------------------------- */
:root {
    --background-color-default: rgb(39 43 48);
    --background-color-default-transparency: rgba(39 43 48 / 75%);
    --background-color-light-transparency: rgba(58 63 68 / 50%);
    --background-panel-default: rgba(200 200 200 / 20%);
    --background-panel-header-default: rgba(62 68 76 / 60%);
    --background-color-success: rgba(98 196 98 / 50%);
    --background-color-success-hover: rgba(79 189 79 / 60%);
    --background-color-info: rgba(91 192 222 / 50%);
    --background-color-info-hover: rgba(70 184 218 / 60%);
    --background-color-warning: rgba(248 148 6 / 50%);
    --background-color-warning-hover: rgba(223 133 5 / 60%);
    --background-color-danger: rgba(238 95 91 / 50%);
    --background-color-danger-hover: rgba(236 72 68 / 60%);
    --background-color-navigation-default-hover: rgb(62 68 76);
    --border-color-success: rgb(85 177 66);
    --border-color-info: rgb(42 199 210);
    --border-color-warning: rgb(208 90 5);
    --border-color-danger: rgb(234 54 79);
    --color-darkmode-default: rgb(78 154 6);
    --border-color-darkmode-default: rgba(78 154 6 / 60%);
    --background-darkmode-color-default: rgb(0 0 0);
    --background-color-darkmode-navigation-default-active: rgba(78 154 6 / 40%);
    --white: rgb(255 255 255);
    --black: rgb(0 0 0);
    --background-body-default: url("https://web-cdn.eveonline.com/aws/eveonline/sso/background.jpg") no-repeat center center fixed;
    --backdrop-filter-default: saturate(1.5) blur(1.5em);
    --font-color-default: rgb(200 200 200);
    --font-family-default: roboto, "Helvetica Neue", sans-serif;
    --font-family-panel-title: georgia, times, serif;
    --font-family-code: "Fira Code", monospace;
    --font-family-code-font-variation: "Fira Code VF", monospace;
}
